<?php

namespace Drupal\o365_outlook_calendar\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\NodeType;
use Drupal\o365_outlook_calendar\OutlookCalendarSaveEventService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Office 365 - Outlook Calendar settings for this site.
 */
class OutlookCalendarSettingsForm extends ConfigFormBase {

  /**
   * The event service.
   *
   * @var \Drupal\o365_outlook_calendar\OutlookCalendarSaveEventService
   */
  protected $eventService;

  /**
   * A list of calendar fields.
   *
   * @var string[]
   */
  protected $calendarFields = [];

  /**
   * Our class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory interface.
   * @param \Drupal\o365_outlook_calendar\OutlookCalendarSaveEventService $eventService
   *   The event service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, OutlookCalendarSaveEventService $eventService) {
    $this->eventService = $eventService;
    $this->calendarFields = $this->eventService->getCalendarFormFields();

    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('config.factory'), $container->get('o365_outlook_calendar.save_event'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'o365_outlook_calendar_outlook_calendar_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['o365_outlook_calendar.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('o365_outlook_calendar.settings');
    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Automatically save events as Outlook Event.'),
      '#description' => $this->t('When enabled user will be presented with a checkbox to save events in their Outlook Calendar.'),
      '#default_value' => $config->get('enabled'),
    ];

    $form['content_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Content type'),
      '#description' => $this->t('Pick the content type you want to enable this functionality on. When you place the "Add node to calendar" block this setting will prevent that block from being shown on other content types then the one selected.'),
      '#options' => static::getNodeTypeOptions(),
      '#default_value' => $config->get('content_type'),
      '#ajax' => [
        'wrapper' => 'content-type-fieldset-wrapper',
        'callback' => '::promptCallback',
      ],
      '#states' => [
        'required' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['fields_fieldset'] = [
      '#type' => 'details',
      '#title' => $this->t('Fields for the selected content type'),
      '#open' => TRUE,
      '#attributes' => ['id' => 'content-type-fieldset-wrapper'],
    ];
    $form['fields_fieldset']['description'] = [
      '#markup' => $this->t('When you have selected a content type you can make a mapping on fields from the content type to the fields in the event created in Outlook Calendar.'),
    ];

    $contentType = $form_state->getValue('content_type');
    if (!$contentType && !empty($config->get('content_type'))) {
      $contentType = $config->get('content_type');
    }

    if (!empty($contentType)) {
      $this->getNodeTypeFormFieldsCheckboxes($form['fields_fieldset'], $contentType);
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if (!empty($form_state->getValue('enabled'))) {
      // We want to reset the config before saving the new values.
      $this->config('o365_outlook_calendar.settings')->delete();

      // Update our config based on form values.
      $config = $this->config('o365_outlook_calendar.settings');
      $config->set('enabled', $form_state->getValue('enabled'));
      $config->set('content_type', $form_state->getValue('content_type'));

      foreach ($this->calendarFields as $key => $calendarField) {
        $config->set($key, $form_state->getValue($key));
      }

      $config->save();
    }
    else {
      // Delete our config.
      $this->config('o365_outlook_calendar.settings')->delete();
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Helper function to populate the content type dropdown.
   *
   * @return array
   *   Dropdown options.
   */
  public static function getNodeTypeOptions() {
    $types = NodeType::loadMultiple();
    $options = [];
    foreach ($types as $type) {
      $options[$type->id()] = $type->label();
    }

    return $options;
  }

  /**
   * Generate the node type fields.
   *
   * @param array $form
   *   The form array. Passed by reference.
   * @param string $contentType
   *   The content type.
   */
  public function getNodeTypeFormFieldsCheckboxes(array &$form, $contentType) {
    $config = $this->config('o365_outlook_calendar.settings');
    $fields = $this->eventService->getNodeTypeFields($contentType);

    foreach ($this->calendarFields as $key => $calendarField) {
      $form[$key] = $calendarField;
      $form[$key]['#type'] = 'select';
      $form[$key]['#options'] = $fields;
      $form[$key]['#default_value'] = $config->get($key);
    }
  }

  /**
   * Callback for the select element.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function promptCallback(array $form, FormStateInterface $form_state) {
    return $form['fields_fieldset'];
  }

}
